package org.wso2.carbon.application.deployer;

import org.wso2.carbon.application.deployer.config.ApplicationConfiguration;

import java.util.ArrayList;
import java.util.HashMap;
import java.io.File;


public class CarbonApplication {

    public static final String AAR_SERVICES = "aar";
    public static final String JAXWS_SERVICES = "jaxws";

    private String appName;
    private String extractedPath;
    private String appFilePath;

    private ArrayList<String> proxyServices;
    private ArrayList<String> endpoints;
    private ArrayList<String> sequences;
    private ArrayList<String> localentries;
    private ArrayList<String> events;
    private ArrayList<String> tasks;

    private HashMap<String, ArrayList> services;

    private ApplicationConfiguration appConfig;

    public CarbonApplication() {
        proxyServices = new ArrayList<String>();
        endpoints = new ArrayList<String>();
        sequences = new ArrayList<String>();
        localentries = new ArrayList<String>();
        events = new ArrayList<String>();
        tasks = new ArrayList<String>();

        services = new HashMap<String, ArrayList>();
        services.put(AAR_SERVICES, new ArrayList<String>());
        services.put(JAXWS_SERVICES, new ArrayList<String>());
    }

    public String getAppName() {
        return appName;
    }

    public void setAppName(String appName) {
        this.appName = appName;
    }

    /**
     * Always retruns the path with a '/' at the end
     * @return - extracted path of the application
     */
    public String getExtractedPath() {
        return extractedPath;
    }

    public void setExtractedPath(String extractedPath) {
        if (!extractedPath.endsWith(File.separator)) {
            extractedPath = extractedPath + File.separator;
        }
        this.extractedPath = extractedPath;
    }

    public ArrayList<String> getProxyServices() {
        return proxyServices;
    }

    public ArrayList<String> getEndpoints() {
        return endpoints;
    }

    public ArrayList<String> getSequences() {
        return sequences;
    }

    public ArrayList<String> getLocalentries() {
        return localentries;
    }

    public ArrayList<String> getEvents() {
        return events;
    }

    public ArrayList<String> getTasks() {
        return tasks;
    }

    public HashMap<String, ArrayList> getServices() {
        return services;
    }

    public void addProxyService(String psFileName) {
        this.proxyServices.add(psFileName);
    }

    public void addSequence(String sequenceFileName) {
        this.sequences.add(sequenceFileName);
    }

    public void addEndpoint(String endpointFileName) {
        this.endpoints.add(endpointFileName);
    }

    public void addLocalEntry(String leFileName) {
        this.localentries.add(leFileName);
    }

    public void addEvent(String eventFileName) {
        this.events.add(eventFileName);
    }

    public void addTask(String taskFileName) {
        this.tasks.add(taskFileName);
    }

    public void addAARService(String aarServiceFileName) {
        this.services.get(AAR_SERVICES).add(aarServiceFileName);
    }

    public void addJAXWSService(String jaxwsServiceFileName) {
        this.services.get(JAXWS_SERVICES).add(jaxwsServiceFileName);
    }

    public boolean isUnusable() {
        return sequences.size() + proxyServices.size() + endpoints.size()
                + services.get(AAR_SERVICES).size() + services.get(JAXWS_SERVICES).size() == 0;
    }

    public ApplicationConfiguration getAppConfig() {
        return appConfig;
    }

    public void setAppConfig(ApplicationConfiguration appConfig) {
        this.appConfig = appConfig;
    }

    public String getAppFilePath() {
        return appFilePath;
    }

    public void setAppFilePath(String appFilePath) {
        this.appFilePath = appFilePath;
    }
}

