package org.wso2.carbon.application.deployer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.application.deployer.config.Component;

import java.util.List;
import java.io.*;

public class DefaultAppDeployer implements AppDeploymentHandler {

    private static final Log log = LogFactory.getLog(DefaultAppDeployer.class);

    private static final String AAR_TYPE = "aar-service";
    private static final String JAXWS_TYPE = "jaxws-service";

    /**
     * Deploy the artifacts which can be deployed through this deployer (Axis2 services and
     * modules).
     *
     * @param carbonApp - store info in this object after deploying
     */
    public void deployArtifacts(CarbonApplication carbonApp) {
        log.info("***Default App Deployer***");

        List<Component> components = carbonApp.getAppConfig().getComponents();
        String artifactPath, destPath;

        for (Component comp : components) {
            if (AAR_TYPE.equals(comp.getType())) {
                destPath = AppDeployerUtils.getAxis2Repo() + File.separator + "axis2services";
                if (isInvalid(comp,  carbonApp.getAppName())) {
                    continue;
                }
                carbonApp.addAARService(comp.getArtifactName());
            } else if (JAXWS_TYPE.equals(comp.getType())) {
                destPath = AppDeployerUtils.getAxis2Repo() + File.separator + "servicejars";
                if (isInvalid(comp,  carbonApp.getAppName())) {
                    continue;
                }
                carbonApp.addJAXWSService(comp.getArtifactName());
            } else {
                continue;
            }

            artifactPath = carbonApp.getExtractedPath() + comp.getId() +
                    File.separator + comp.getArtifactName();

            AppDeployerUtils.createDir(destPath);
            AppDeployerUtils.copyFile(artifactPath, destPath + File.separator +
                    comp.getArtifactName());
        }
    }

    private boolean isInvalid(Component comp, String appName) {
        if (comp.getArtifactName() == null) {
            log.error("Invalid Component " + comp.getId() + " found in Application : " +
                    appName + ". Artifact Name not found.");
            return true;
        }
        return false;
    }

}
