/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.bridge;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionContext;
import java.util.Enumeration;

/**
 * The HttpSession request that is used withing Carbon. This is set in the
 * {@link org.wso2.carbon.bridge.BridgeServlet} so that Carbon can take control over how attributes
 * are set in the servlet API. This has been adopted so that ClassNotFoundExceptions &
 * ClassCastExceptions can be avoided when Carbon is deployed within App server such as WebLogic
 * which serializes HttpSession attriubutes & HttpRequest attributes.
 */
public class CarbonHttpSession implements HttpSession {

    private HttpSession delegate;

    public CarbonHttpSession(HttpSession delegate) {
        this.delegate = delegate;
    }

    /**
     * Overriden
     *
     * @param s Key
     * @return attribute
     */
    public Object getAttribute(String s) {
        Object attribute = delegate.getAttribute(s);
        if (attribute instanceof CarbonAttributeWrapper) {
            return ((CarbonAttributeWrapper) attribute).getObject();
        }
        return attribute;
    }

    /**
     * Overriden
     *
     * @param s key
     * @return value
     */
    public Object getValue(String s) {
         Object attribute = delegate.getAttribute(s);
        if (attribute instanceof CarbonAttributeWrapper) {
            return ((CarbonAttributeWrapper) attribute).getObject();
        }
        return attribute;
    }

    /**
     * Overriden
     *
     * @param s key
     * @param o attribute
     */
    public void setAttribute(String s, Object o) {
        delegate.setAttribute(s, new CarbonAttributeWrapper(o));
    }

    /**
     * Overriden
     *
     * @param s key
     * @param o value
     */
    public void putValue(String s, Object o) {
        delegate.putValue(s, new CarbonAttributeWrapper(o));
    }

    public void invalidate() {
        delegate.invalidate();
    }

    // ---------------------------------------------------------------------------------------------

    public void removeAttribute(String s) {
        delegate.removeAttribute(s);
    }

    public void removeValue(String s) {
        delegate.removeValue(s);
    }

    public long getCreationTime() {
        return delegate.getCreationTime();
    }

    public String getId() {
        return delegate.getId();
    }

    public long getLastAccessedTime() {
        return delegate.getLastAccessedTime();
    }

    public ServletContext getServletContext() {
        return delegate.getServletContext();
    }

    public void setMaxInactiveInterval(int i) {
        delegate.setMaxInactiveInterval(i);
    }

    public int getMaxInactiveInterval() {
        return delegate.getMaxInactiveInterval();
    }

    public HttpSessionContext getSessionContext() {
        return delegate.getSessionContext();
    }

    public Enumeration getAttributeNames() {
        return delegate.getAttributeNames();
    }

    public String[] getValueNames() {
        return delegate.getValueNames();
    }

    public boolean isNew() {
        return delegate.isNew();
    }
}
