/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.server;

import org.apache.catalina.Context;
import org.apache.catalina.Engine;
import org.apache.catalina.Host;
import org.apache.catalina.realm.MemoryRealm;
import org.apache.catalina.startup.Embedded;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.server.transports.TransportManager;
import org.wso2.carbon.utils.CarbonUtils;

import java.io.File;
import java.util.Random;

/**
 *
 */
public class TomcatServer {

    private static Log log = LogFactory.getLog(TomcatServer.class);
    public static final Embedded embedded = TomcatServerCache.getEmbedded();

    public TomcatServer() {
    }

    public void  start() throws Exception {
        Engine engine;

        String catalinaHome = CarbonUtils.getCatalinaHome();

        embedded.setCatalinaHome(catalinaHome);
        String webappsDir = CarbonUtils.getTomcatWebappDirPath();
        String docsDir = CarbonUtils.getTomcatDocsDir();

        // set the memory realm
        MemoryRealm memRealm = new MemoryRealm();
        embedded.setRealm(memRealm);

        // Create an engine
        engine = embedded.createEngine();
        embedded.addEngine(engine);
        TransportManager.init();
        engine.setName("carbonEngine");

        String hostName = "carbonHost";
        engine.setDefaultHost(hostName);

        // Create a default virtual host
        Host defaultHost = embedded.createHost(hostName, webappsDir);
        engine.addChild(defaultHost);

        File docs = new File(docsDir);
        if (docs.exists()) {
            Context docsContext = embedded.createContext("/docs", docs.getAbsolutePath());
            defaultHost.addChild(docsContext);
        }

        defaultHost.setDeployOnStartup(true);
        defaultHost.setAutoDeploy(true);

        embedded.setUseNaming(true);

        // Start the embedded server
        embedded.start();
        // Start the embedded server
        try {
            TomcatRepositoryListener tomcatRepositoryListener =
                    new TomcatRepositoryListener(embedded, defaultHost,
                                                 new File(webappsDir).getAbsolutePath());
            new Thread(tomcatRepositoryListener).start();
        } catch (Exception e) {
            log.error("Cannot start Tomcat repository listener", e);
        }
    }


    public void stop() throws Exception {
        embedded.stop();
    }
}
