package org.wso2.carbon.ui;

import java.rmi.RemoteException;

import org.wso2.carbon.core.common.AuthenticationException;

/**
 * This is the interface of the Authenticator to Carbon UI framework. Implement
 * this interface in a bundle and drop-in to the framework and it will be
 * automatically called. The authenticator has the freedom to decide how it does
 * the authentication.
 * 
 * When an authentication request is received by the framework each
 * authenticator will be checked to see who handles the request, i.e. isHandle()
 * method of each authenticator will be called in the priority order given by
 * the getPriority() method. Highest priority authenticator that returns true
 * for the isHandle() method will be given the responsibility of authentication.
 * 
 * Priority is defined as higher the number higher the priority.
 */
public interface CarbonUIAuthenticator {

    /**
     * Does this authenticator handle this type of requests
     * 
     * @param request
     * @return
     * @throws RemoteException
     */
    public boolean isHandle(Object object);

    /**
     * Authenticates user based on the credentials contained in the request.
     * 
     * @param request
     * @return
     * @throws AuthenticationException
     */
    public boolean authenticate(Object object) throws AuthenticationException;

    /**
     * Invalidates the authenticated session for the logged in user.
     * 
     * @param request
     */
    public void unauthenticate(Object object) throws Exception;

    /**
     * Gets the priority of this authenticator in the framework
     * 
     * @return
     */
    public int getPriority();

    /**
     * The name of the Authenticator
     * 
     * @return
     */
    public String getAuthenticatorName();
    
    /**
	 * By default all the authenticators found in the system are enabled. Can
	 * use this property to control default behavior.
	 * 
	 * @return
	 */
	public boolean isDisabled();
}
