package org.wso2.carbon.ui;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.authenticator.proxy.AuthenticationAdminClient;
import org.wso2.carbon.core.common.AuthenticationException;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.utils.ServerConfiguration;
import org.wso2.carbon.utils.ServerConstants;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.rmi.RemoteException;

public class DefaultCarbonAuthenticator implements CarbonUIAuthenticator {

	protected static final Log log = LogFactory
			.getLog(DefaultCarbonAuthenticator.class);
	protected static final String AUTHENTICATION_ADMIN_SERVICE = "AuthenticationAdminService";
	private static final int DEFAULT_PRIORITY_LEVEL = 5;
	private static final String AUTHENTICATOR_NAME = "Authenticators.DefaultCarbonAuthenticator";
	private static final String AUTHENTICATOR_PRIOROTY = AUTHENTICATOR_NAME + ".Priority";
	private static final String AUTHENTICATOR_DISABLED = AUTHENTICATOR_NAME + ".Disabled";

	public boolean isHandle(Object object) {
		// try to authenticate any request that comes
		// least priority authenticator
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean authenticate(Object object) throws AuthenticationException {
		HttpServletRequest request = (HttpServletRequest) object;
		String userName = request.getParameter("username");
		String password = request.getParameter("password");
		try {
			return authenticate(request, userName, password);
		} catch (RemoteException e) {
			throw new AuthenticationException(e.getMessage(), e);
		}
	}

	/**
     *
     */
	public void unauthenticate(Object object) throws Exception {
		HttpSession session = ((HttpServletRequest) object).getSession();
		ServletContext servletContext = session.getServletContext();
		ConfigurationContext configContext = (ConfigurationContext) servletContext
				.getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

		String backendServerURL = CarbonUIUtil.getServerURL(servletContext,
				session);
		try {
			String cookie = (String) session
					.getAttribute(ServerConstants.ADMIN_SERVICE_AUTH_TOKEN);
			getAuthenticationAdminProxy(session, configContext,
					backendServerURL, cookie).logout();
		} catch (Exception ignored) {
			String msg = "Configuration context is null.";
			log.error(msg);
			throw new Exception(msg);
		}
	}

	protected boolean authenticate(HttpServletRequest request, String userName,
			String password) throws RemoteException {
		try {

			ServletContext servletContext = request.getSession()
					.getServletContext();
			ConfigurationContext configContext = (ConfigurationContext) servletContext
					.getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

			if (configContext == null) {
				String msg = "Configuration context is null.";
				log.error(msg);
				throw new RemoteException(msg);
			}
			// Obtain the back-end server URL from the request. If not obtain it
			// from the http
			// session and then from the ServletContext.
			HttpSession session = request.getSession();
			String backendServerURL = request.getParameter("backendURL");
			if (backendServerURL == null) {
				backendServerURL = CarbonUIUtil.getServerURL(servletContext,
						request.getSession());
			}

			// Back-end server URL is stored in the session, even if it is an
			// incorrect one. This
			// value will be displayed in the server URL text box. Usability
			// improvement.
			session.setAttribute(CarbonConstants.SERVER_URL, backendServerURL);

			String cookie = (String) session
					.getAttribute(ServerConstants.ADMIN_SERVICE_AUTH_TOKEN);
			AuthenticationAdminClient proxy = getAuthenticationAdminProxy(
					session, configContext, backendServerURL, cookie);

			String userNameWithDomain = userName;
			String domainName = (String) request
					.getAttribute(RegistryConstants.TENANT_DOMAIN);
			if (domainName != null) {
				userNameWithDomain += "@" + domainName;
			}

			boolean isLogged = proxy.login(userNameWithDomain, password,
					request.getRemoteAddr());

			return isLogged;

		} catch (AxisFault axisFault) {
			throw axisFault;
		} catch (RemoteException e) {
			throw e;
		} catch (Exception e) {
			throw new AxisFault("Exception occured", e);
		}
	}

	/**
	 * Returns the priority
	 */
	public int getPriority() {
		ServerConfiguration serverConfig = ServerConfiguration.getInstance();
		String priority = serverConfig.getFirstProperty(AUTHENTICATOR_PRIOROTY);
		if (priority != null && priority.length() > 0) {
			return Integer.parseInt(priority);
		}
		return DEFAULT_PRIORITY_LEVEL;
	}

	public String getAuthenticatorName() {
		return AUTHENTICATOR_NAME;
	}

	protected AuthenticationAdminClient getAuthenticationAdminProxy(
			HttpSession session, ConfigurationContext configContext,
			String backendServerURL, String cookie) throws AxisFault {
		return new AuthenticationAdminClient(configContext, backendServerURL,
				cookie, session, true);
	}

    public boolean isDisabled() {
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        String isDisabled = serverConfig.getFirstProperty(AUTHENTICATOR_DISABLED);
        if (isDisabled != null && isDisabled.length() > 0) {
            return Boolean.getBoolean(isDisabled);
        }
        return false;
    }
}
