/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.user.core;

import java.util.Map;

import org.wso2.carbon.user.core.claim.ClaimManager;
import org.wso2.carbon.user.core.claim.ClaimMapping;
import org.wso2.carbon.user.core.config.RealmConfiguration;
import org.wso2.carbon.user.core.profile.ProfileConfiguration;
import org.wso2.carbon.user.core.profile.ProfileConfigurationManager;

/**
 * The Realm of the WSO2 user Kernel.
 * 
 * The realm represents a user store. This is a collection of interfaces.
 * 
 * To enable WSO2 platform with a clustom realm, implement this interface and
 * add the class to the class path. Provide the class name in the configuration
 * file and the framework will pick the new realm code.
 */
public interface UserRealm {

    /**
     * Initialize the realm.
     * 
     * @param configBean - Configuration details of the realm
     * @throws UserStoreException
     */
    public void init(RealmConfiguration configBean, Map<String, ClaimMapping> claimMapping,
            Map<String, ProfileConfiguration> profileConfigs, int tenantId) throws UserStoreException;
    
    /**
     * Get the AuthorizationReader of the system
     * @return The AuthorizationReader the system
     * @throws UserStoreException
     */
    public AuthorizationManager getAuthorizationManager() throws UserStoreException;

    /**
     * Get the UserStoreManager of the system.
     * @return The UserStoreManager of the system
     * @throws UserStoreException
     */
    public UserStoreManager getUserStoreManager() throws UserStoreException;

    
    public ClaimManager getClaimManager()  throws UserStoreException;
    /**
     * Get the ProfileConfigurationManager of the system.
     * @return The ProfileConfigurationManager of the system
     * @throws UserStoreException
     */ 
    public ProfileConfigurationManager getProfileConfigurationManager() throws UserStoreException;
     
    /**
     * Clean up the system. Clean up dead data.
     * @throws UserStoreException
     */
    public void cleanUp() throws UserStoreException;
    
    public RealmConfiguration getRealmConfiguration() throws UserStoreException;
}
