package org.wso2.carbon.user.core.claim;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.sql.DataSource;

import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.claim.dao.ClaimDAO;

public class DefaultClaimManager implements ClaimManager {

    private Map<String, ClaimMapping> claimMapping = new ConcurrentHashMap<String, ClaimMapping>();
    private ClaimDAO claimDAO = null;
    int tenantId;

    /**
     * 
     * @param claimMapping
     */
    public DefaultClaimManager(Map<String, ClaimMapping> claimMapping, DataSource dataSource,
            int tenantId) {
        this.claimMapping = new ConcurrentHashMap<String, ClaimMapping>();
        this.claimMapping.putAll(claimMapping);
        this.claimDAO = new ClaimDAO(dataSource, tenantId);
    }

    /**
     * {@inheritDoc}
     */
    public String getAttributeName(String claimURI) throws UserStoreException {
        ClaimMapping mapping = claimMapping.get(claimURI);
        if (mapping != null) {
            return mapping.getMappedAttribute();
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public Claim getClaim(String claimURI) throws UserStoreException {
        ClaimMapping mapping = claimMapping.get(claimURI);
        if (mapping != null) {
            return mapping.getClaim();
        }
        return null;
    }

    public ClaimMapping getClaimMapping(String claimURI) throws UserStoreException {
        return claimMapping.get(claimURI);
    }

    /**
     * {@inheritDoc}
     */
    public Claim[] getAllSupportClaimsByDefault() throws UserStoreException {
        List<Claim> claimList = null;
        claimList = new ArrayList<Claim>();
        Iterator<Entry<String, ClaimMapping>> iterator = claimMapping.entrySet().iterator();

        for (; iterator.hasNext();) {
            ClaimMapping claimMapping = iterator.next().getValue();
            Claim claim = claimMapping.getClaim();
            if (claim.isSupportedByDefault()) {
                claimList.add(claim);
            }
        }

        return claimList.toArray(new Claim[claimList.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public Claim[] getAllClaims() throws UserStoreException {
        List<Claim> claimList = null;
        claimList = new ArrayList<Claim>();
        Iterator<Entry<String, ClaimMapping>> iterator = claimMapping.entrySet().iterator();

        for (; iterator.hasNext();) {
            ClaimMapping claimMapping = iterator.next().getValue();
            Claim claim = claimMapping.getClaim();
            claimList.add(claim);
        }
        return claimList.toArray(new Claim[claimList.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public Claim[] getAllClaims(String dialectUri) throws UserStoreException {
        List<Claim> claimList = null;
        claimList = new ArrayList<Claim>();
        Iterator<Entry<String, ClaimMapping>> iterator = claimMapping.entrySet().iterator();

        for (; iterator.hasNext();) {
            ClaimMapping claimMapping = iterator.next().getValue();
            Claim claim = claimMapping.getClaim();
            if (claim.getDialectURI().equals(dialectUri)) {
                claimList.add(claim);
            }
        }
        return claimList.toArray(new Claim[claimList.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public Claim[] getAllRequiredClaims() throws UserStoreException {
        List<Claim> claimList = null;
        claimList = new ArrayList<Claim>();
        Iterator<Entry<String, ClaimMapping>> iterator = claimMapping.entrySet().iterator();

        for (; iterator.hasNext();) {
            Claim claim = (Claim) iterator.next().getValue().getClaim();
            if (claim.isRequired()) {
                claimList.add(claim);
            }
        }

        return claimList.toArray(new Claim[claimList.size()]);
    }

    public String[] getAllClaimUris() throws UserStoreException {
        return claimMapping.keySet().toArray(new String[claimMapping.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public void addNewClaimMapping(ClaimMapping mapping) throws UserStoreException {
        if (mapping != null && mapping.getClaim() != null) {
            if (!claimMapping.containsKey(mapping.getClaim().getClaimUri())) {
                claimMapping.put(mapping.getClaim().getClaimUri(), mapping);
                claimDAO.addClaimMapping(mapping);
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public void deleteClaimMapping(ClaimMapping mapping) throws UserStoreException {
        if (mapping != null && mapping.getClaim() != null) {
            if (claimMapping.containsKey(mapping.getClaim().getClaimUri())) {
                claimMapping.remove(mapping.getClaim().getClaimUri());
                claimDAO.deleteClaimMapping(mapping);
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public void updateClaimMapping(ClaimMapping mapping) throws UserStoreException {
        if (mapping != null && mapping.getClaim() != null) {
            if (claimMapping.containsKey(mapping.getClaim().getClaimUri())) {
                claimMapping.put(mapping.getClaim().getClaimUri(), mapping);
                claimDAO.updateClaim(mapping);
            }
        }
    }
}
