/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.user.core.config;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * The object representing the realm configuration.
 */
public class RealmConfiguration {

    private String userStoreClass = null;
    private String authorizationManagerClass = null;
    private String adminRoleName = null;
    private String adminUserName = null;
    private String adminPassword = null;
    private String everyOneRoleName = null;
    private Map<String, String> userStoreProperties = new HashMap<String, String>();
    private Map<String, String> authzProperties = new HashMap<String, String>();
    private Map<String, String> realmProperties = new HashMap<String, String>();
    private Map<String, Object> properties = new HashMap<String, Object>();
    boolean isReadOnly = true;
    private int tenantId;
    private Date persistedTimestamp;
    private int maxUserListLength = -1;

    public RealmConfiguration() {
        tenantId = 0;
    }
   
    public String getAuthorizationPropertyValue(String propertyName) {
        return authzProperties.get(propertyName);
    }

    public String getRealmProperty(String propertyName) {
        return realmProperties.get(propertyName);
    }

    public String getUserStoreProperty(String propertyName){
        return userStoreProperties.get(propertyName);
    }
    
    public String getAdminRoleName() {
        return adminRoleName;
    }

    public String getAdminUserName() {
        return adminUserName;
    }

    public String getAdminPassword() {
        return adminPassword;
    }
    
    public String getEveryOneRoleName() {
        return everyOneRoleName;
    }
   
    public String getAuthorizationManagerClass() {
        return authorizationManagerClass;
    }
        
      
    public String getAuthorizationManagerProperty(String key){
        return authzProperties.get(key); 
    }
    
    public String getUserStoreClass() {
        return userStoreClass;
    }


    public Map<String, String> getUserStoreProperties() {
        return userStoreProperties;
    }


    public Map<String, String> getAuthzProperties() {
        return authzProperties;
    }


    public Map<String, String> getRealmProperties() {
        return realmProperties;
    }
    
    
    public boolean isReadOnly() {
        return isReadOnly;
    }

    public int getMaxUserListLength() {
        return maxUserListLength;
    }
    
    public void setAdminRoleName(String adminRoleName) {
        this.adminRoleName = adminRoleName;
    }

    public void setEveryOneRoleName(String everyOneRoleName) {
        this.everyOneRoleName = everyOneRoleName;
    }

    public void setAuthzProperties(Map<String, String> authzProperties) {
        this.authzProperties = authzProperties;
    }

    public void setRealmProperties(Map<String, String> realmProperties) {
        this.realmProperties = realmProperties;
    }

    public void setAuthorizationManagerClass(String authorizationManagerClass) {
        this.authorizationManagerClass = authorizationManagerClass;
    }


    public void setUserStoreClass(String userStoreClass) {
        this.userStoreClass = userStoreClass;
    }


    public void setUserStoreProperties(Map<String, String> userStoreProperties) {
        this.userStoreProperties = userStoreProperties;
    }

    public void setReadOnly(boolean isReadOnly) {
        this.isReadOnly = isReadOnly;
    }

    public void setMaxUserListLength(int maxUserListLength) {
        this.maxUserListLength = maxUserListLength;
    }


    // two public setter methods used for external editing
    public void setAdminUserName(String adminUserName) {
        this.adminUserName = adminUserName;
    }

    public void setAdminPassword(String adminPassword) {
        this.adminPassword = adminPassword;
    }

    public int getTenantId() {
        return tenantId;
    }

    public void setTenantId(int tenantId) {
        this.tenantId = tenantId;
    }

    public Date getPersistedTimestamp() {
        return persistedTimestamp;
    }

    public void setPersistedTimestamp(Date persistedTimestamp) {
        this.persistedTimestamp = persistedTimestamp;
    }

    public Map<String, Object> getProperties() {
        return properties;
    }

    public void setProperties(Map<String, Object> properties) {
        this.properties = properties;
    }

}
