package org.wso2.carbon.user.core.ldap;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.config.RealmConfiguration;

public class LDAPConnectionContext {
    
    private Hashtable environment;

    private static Log log = LogFactory.getLog(LDAPConnectionContext.class);

    public LDAPConnectionContext(RealmConfiguration realmConfig) {
        
        String connectionURL = realmConfig.getUserStoreProperty(LDAPConstants.CONNECTION_URL);
        String connectionName = realmConfig.getUserStoreProperty(LDAPConstants.CONNECTION_NAME);
        String connectionPassword = realmConfig
                .getUserStoreProperty(LDAPConstants.CONNECTION_PASSWORD);
        
        if (log.isDebugEnabled()) {
            log.debug("Connection Name :: " + realmConfig.getUserStoreProperty(connectionURL) + ","
                    + "Connection Password :: " + connectionPassword + "," + "Connection URL :: "
                    + connectionURL);
        }
        
        environment = new Hashtable();
        
        environment.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        environment.put(Context.SECURITY_AUTHENTICATION, "simple");

        if (connectionName != null) {
            environment.put(Context.SECURITY_PRINCIPAL, connectionName);
        }

        if (connectionPassword != null) {
            environment.put(Context.SECURITY_CREDENTIALS, connectionPassword);
        }

        if (connectionURL != null){
            environment.put(Context.PROVIDER_URL, connectionURL);
        }

    }

    public DirContext getContext() throws UserStoreException {
        DirContext context = null;
        try {
            context = new InitialDirContext(environment);
        } catch (NamingException e) {
            log.error("Error obtaining connection. " + e.getMessage(), e);
            log.error("Trying again to get connection.");

            try {
                context = new InitialDirContext(environment);
            } catch (Exception e1) {
                log.error("Error obtaining connection for the second time" + e.getMessage(), e);
                throw new UserStoreException("Error obtaining connection. " + e.getMessage(), e);
            }

        }
        return (context);

    }

    public void updateCredential(String connectionPassword){
        // update the password otherwise it is not possible to connect
        // again if admin password changed
        this.environment.put(Context.SECURITY_CREDENTIALS, connectionPassword);
    }
}
