package org.wso2.carbon.user.core.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.config.RealmConfiguration;
import org.wso2.carbon.user.core.jdbc.JDBCRealmConstants;

public class DatabaseUtil {

    private static Log log = LogFactory.getLog(DatabaseUtil.class);
    
    public static DataSource createRealmDataSource(RealmConfiguration realmConfig){
         BasicDataSource ds = new BasicDataSource();
         ds.setDriverClassName(realmConfig.getRealmProperty(JDBCRealmConstants.DRIVER_NAME));
         ds.setUrl(realmConfig.getRealmProperty(JDBCRealmConstants.URL));
         ds.setUsername(realmConfig.getRealmProperty(JDBCRealmConstants.USER_NAME));
         ds.setPassword(realmConfig.getRealmProperty(JDBCRealmConstants.PASSWORD));
         ds.setMaxActive(Integer.parseInt(realmConfig.getRealmProperty(JDBCRealmConstants.MAX_ACTIVE)));
         ds.setMinIdle(Integer.parseInt(realmConfig.getRealmProperty(JDBCRealmConstants.MIN_IDLE)));
         ds.setMaxWait(Integer.parseInt(realmConfig.getRealmProperty(JDBCRealmConstants.MAX_WAIT)));
         return ds;
    }


    public static String[] getStringValuesFromDatabase(Connection dbConnection, String sqlStmt, Object... params)
            throws UserStoreException {
        String[] values = new String[0];
        PreparedStatement prepStmt = null;
        ResultSet rs = null;
        try {
            prepStmt = dbConnection.prepareStatement(sqlStmt);
            if (params != null && params.length > 0) {
                for (int i = 0; i < params.length; i++) {
                    Object param = params[i];
                    if (param == null) {
                        throw new UserStoreException("Null data provided.");
                    } else if (param instanceof String) {
                        prepStmt.setString(i + 1, (String)param);
                    } else if (param instanceof Integer) {
                        prepStmt.setInt(i + 1, (Integer)param);
                    }
                }
            }
            rs = prepStmt.executeQuery();
            List<String> lst = new ArrayList<String>();
            while (rs.next()) {
                String name = rs.getString(1);
                lst.add(name);
            }
            if (lst.size() > 0) {
                values = lst.toArray(new String[lst.size()]);
            }

            return values;
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            log.error("Using sql : " + sqlStmt);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            //DatabaseUtil.closeAllConnections(dbConnection, rs, prepStmt);
        }
    }
    
    
    public static int getIntegerValueFromDatabase(Connection dbConnection, String sqlStmt,
            Object... params) throws UserStoreException {
        PreparedStatement prepStmt = null;
        ResultSet rs = null;
        int value = -1;
        try {
            prepStmt = dbConnection.prepareStatement(sqlStmt);
            if (params != null && params.length > 0) {
                for (int i = 0; i < params.length; i++) {
                    Object param = params[i];
                    if (param == null) {
                        throw new UserStoreException("Null data provided.");
                    } else if (param instanceof String) {
                        prepStmt.setString(i + 1, (String) param);
                    } else if (param instanceof Integer) {
                        prepStmt.setInt(i + 1, (Integer) param);
                    }
                }
            }
            rs = prepStmt.executeQuery();
            if (rs.next()) {
                value = rs.getInt(1);
            }
            return value;
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            log.error("Using sql : " + sqlStmt);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            
        }
    }

    public static void udpateUserRoleMappingInBatchMode(Connection dbConnection, String sqlStmt,
            Object... params) throws UserStoreException {
        PreparedStatement prepStmt = null;
        boolean localConnection = false;
        try {

            prepStmt = dbConnection.prepareStatement(sqlStmt);
            int batchParamIndex = -1;
            if (params != null && params.length > 0) {
                for (int i = 0; i < params.length; i++) {
                    Object param = params[i];
                    if (param == null) {
                        throw new UserStoreException("Null data provided.");
                    } else if (param instanceof String[]) {
                        batchParamIndex = i;
                    } else if (param instanceof String) {
                        prepStmt.setString(i + 1, (String)param);
                    } else if (param instanceof Integer) {
                        prepStmt.setInt(i + 1, (Integer)param);
                    }
                }
            }
            if (batchParamIndex != -1) {
                String[] values = (String[])params[batchParamIndex];
                for (String value : values) {
                    prepStmt.setString(batchParamIndex + 1, value);
                    prepStmt.addBatch();
                }
            }

            int[] count = prepStmt.executeBatch();
            if (log.isDebugEnabled()) {
                log.debug("Executed a batch update. Querry is : " + sqlStmt + ": and result is"
                        + Arrays.toString(count));
            }
            if (localConnection) {
                dbConnection.commit();
            }
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            log.error("Using sql : " + sqlStmt);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            if (localConnection) {
                DatabaseUtil.closeAllConnections(dbConnection);
            }
            DatabaseUtil.closeAllConnections(null, prepStmt);
        }
    }
    
    public static void updateDatabase(Connection dbConnection, String sqlStmt, Object... params)
            throws UserStoreException {
        PreparedStatement prepStmt = null;
        try {
            prepStmt = dbConnection.prepareStatement(sqlStmt);
            if (params != null && params.length > 0) {
                for (int i = 0; i < params.length; i++) {
                    Object param = params[i];
                    if (param == null) {
                        throw new UserStoreException("Null data provided.");
                    } else if (param instanceof String) {
                        prepStmt.setString(i + 1, (String)param);
                    } else if (param instanceof Integer) {
                        prepStmt.setInt(i + 1, (Integer)param);
                    } else if (param instanceof Short) {
                        prepStmt.setShort(i + 1, (Short)param);
                    }
                }
            }
            int count = prepStmt.executeUpdate();
            if (log.isDebugEnabled()) {
                log.debug("Executed querry is " + sqlStmt + " and number of updated rows :: "
                        + count);
            }
        } catch (SQLException e) {
            log.error("Error! " + e.getMessage(), e);
            log.error("Using sql : " + sqlStmt);
            throw new UserStoreException("Error! " + e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(null, prepStmt);
        }
    }

    public static void closeConnection(Connection dbConnection) throws UserStoreException {
        try {
            if (dbConnection != null) {
                dbConnection.close();
            }
        } catch (SQLException e) {
            log.error("Database error. Could not close connections - " + e.getMessage(), e);
            throw new UserStoreException("Database error. Could not close connections.", e);
        }
    }

    public static void closeAllConnections(Connection dbConnection, PreparedStatement... prepStmts)
            throws UserStoreException {
        try {
            if (prepStmts != null && prepStmts.length > 0) {
                for (PreparedStatement stmt : prepStmts) {
                    if (stmt != null) {
                        stmt.close();
                    }
                }
            }
            if (dbConnection != null) {
                dbConnection.close();
            }
        } catch (SQLException e) {
            log.error("Database error. Could not close connections - " + e.getMessage(), e);
            throw new UserStoreException("Database error. Could not close connections.", e);
        }
    }

    public static void closeAllConnections(Connection dbConnection, ResultSet rs,
            PreparedStatement... prepStmts) throws UserStoreException {
        try {
            if (rs != null) {
                rs.close();
            }

            if (prepStmts != null && prepStmts.length > 0) {
                for (PreparedStatement stmt : prepStmts) {
                    if (stmt != null) {
                        stmt.close();
                    }
                }
            }

            if (dbConnection != null) {
                dbConnection.close();
            }
        } catch (SQLException e) {
            log.error("Database error. Could not close connections  - " + e.getMessage(), e);
            throw new UserStoreException("Database error. Could not close connections.", e);
        }
    }

    public static void closeAllConnections(Connection dbConnection, ResultSet rs1, ResultSet rs2,
            PreparedStatement... prepStmts) throws UserStoreException {
        try {
            if (rs1 != null) {
                rs1.close();
            }

            if (rs2 != null) {
                rs2.close();
            }

            if (prepStmts != null && prepStmts.length > 0) {
                for (PreparedStatement stmt : prepStmts) {
                    if (stmt != null) {
                        stmt.close();
                    }
                }
            }

            if (dbConnection != null) {
                dbConnection.close();
            }
        } catch (SQLException e) {
            log.error("Database error. Could not close connections  - " + e.getMessage(), e);
            throw new UserStoreException("Database error. Could not close connections.", e);
        }
    }

}
